# 
# test_application.rb
# ===================
# 
# Test the Ptero::Application class
# 

require 'ptero'
require 'minitest/autorun'
require 'mute'


class TestApplication < MiniTest::Test
  
  def setup
    Dir.chdir __dir__
    Mute::IO.capture_stdout do
      @app = Ptero::Application.create('tester')
    end
    Dir.chdir @app.dir
  end
  
  def test_application_creates_and_destroys
    Dir.chdir __dir__ do
      Mute::IO.capture_stdout do
        app = Ptero::Application.create('appname')
        assert File.directory?('appname')
        app.destroy
        refute File.exist?('appname')
      end
    end
  end
  
  def test_app_test_php
    php_works = `php -r'echo "hello";'` == 'hello'
    assert_equal php_works, @app.test_php
  end
  
  def test_get_composer_has_composer
    refute @app.has_composer?
    Mute::IO.capture_stdout do
      @app.get_composer
    end
    assert @app.has_composer?
    assert_raises Ptero::Exception::ApplicationException do
      Mute::IO.capture_stdout do
        @app.get_composer
      end
    end
    @app.remove_composer
    refute @app.has_composer?

    assert_raises Ptero::Exception::ApplicationException do
      @app.remove_composer
    end

  end

  def test_install_dependencies
    Mute::IO.capture_stdout do
      @app.get_composer
      @app.install_dependencies
    end
    assert File.directory? 'vendor'
    assert @app.dependencies_installed?

    @app.remove_dependencies
    refute File.exist? 'vendor'
    refute @app.dependencies_installed?
  end
  
  def test_verify_application
    assert @app.verify
    Mute::IO.capture_stdout do
      a = Ptero::Application.create('throwaway')
      assert a.verify
      a.destroy
      refute a.verify
    end
    
    Mute::IO.capture_stdout do
      assert_raises Ptero::Exception::ApplicationException do
        app = Ptero::Application.app_for('throwaway')
      end
    end
  end
  
  def test_application_generate
    Mute::IO.capture_stdout do
      ['controller','model','javascript'].each do |str|
        g = Ptero::Generator.const_get("#{str[0].upcase}#{str[1,str.length-1]}Generator".intern).new('review')
        @app.generate(str,'review')
        assert g.generated?
        
        @app.reload(str,'review')
        assert g.generated?
        
        @app.remove(str,'review')
        refute g.generated?
      end
    end
  end
  
  def test_generated
    ['controller','model','javascript'].each do |str|
      name = 'exams'
      gen = Ptero::Generator.const_get("#{str[0].upcase}#{str[1,str.length-1]}Generator".intern).new(name)
      Mute::IO.capture_stdout do
        refute gen.generated?
        refute @app.generated? gen.class, name
        refute @app.generated? str, name
        
        gen.generate
        
        assert gen.generated?
        assert @app.generated? gen.class, name
        assert @app.generated? str, name
        
        gen.remove
        

        refute gen.generated?
        refute @app.generated? gen.class, name
        refute @app.generated? str, name
      end
    end
  end
  
  def teardown
    @app.destroy
  end
  
  
end