require 'minitest/autorun'
require 'mute'
require 'ptero'
require 'yaml'
require 'erubis'
 
 
class TestGenerators < MiniTest::Test

  
  def test_generators_should_generate
    @generators.each do |gen|
      type = gen.to_s.split('::').last.downcase
      params = @fixture['generators'][type]['params']
      g = gen.new(*params)
      assert File.exist? g.template_path
      
      # Test #generate
      output = Mute::IO.capture_stdout do
        g.generate
        assert File.exist? g.location
        assert g.generated?
      end
      assert_equal "GENERATE - #{g.to_s}".green, output.split("\n").first
      
      
      # Test #reload
      output = Mute::IO.capture_stdout do
        g.reload
        assert File.exist? g.location
        assert g.generated?
      end
      assert_equal "RELOAD - #{g.to_s}".blue, output.split("\n").first
      
      # Test #remove
      output = Mute::IO.capture_stdout do
        g.remove
        refute File.exist? g.location
        refute g.generated?
      end
      assert_equal "REMOVE - #{g.to_s}".red, output.split("\n").first
      
    end
      
  end
  
  def test_generators_should_produce_content
    @generators.each do |gen|
      type = gen.to_s.split('::').last.downcase
      params = @fixture['generators'][type]['params']
      g = gen.new(*params)
      
      temp_content = File.read(g.template_path)
      eruby = Erubis::Eruby.new(temp_content)
      assert_equal g.content, eruby.evaluate(g.content_params)
    end
  end
  
  def test_generators_should_complain
    @generators.each do |gen|
      type = gen.to_s.split('::').last.downcase
      params = @fixture['generators'][type]['params']
      g = gen.new(*params)
      
      assert_raises(Ptero::Exception::GeneratorException) do
        g.remove
      end
      Mute::IO.capture_stdout do
        g.generate
      end
      assert_raises(Ptero::Exception::GeneratorException) do
        g.generate
      end
      Mute::IO.capture_stdout do
        g.remove
      end
    end
  end
  
  def setup
   app
   generators
   fixture
  end
 
 def app
   Dir.chdir __dir__
   Mute::IO.capture_stdout do
     @app = Ptero::Application.create('app')
   end
   Dir.chdir @app.dir
 end
 
 def generators
   Dir.glob("#{__dir__}/../lib/ptero/generators/*generator.rb").each do |location|
     require location
   end
   
   @generators = Ptero::Generator.constants.select { |sym| sym.match /(\w+)Generator/ }.map { |sym| Ptero::Generator.const_get(sym) }
 end
 
 def fixture
   @fixture = YAML.load(File.read("#{__dir__}/fixtures/test_generators_fixtures.yaml"))
 end
 
 def teardown
   @app.destroy
 end
 
 
end